#!/usr/bin/perl
use strict;
use warnings;
use Getopt::Long;

my %opt = ();
my %legend = (
	"pattern_list=s"                        =>      "list of query patterns",
	"place_file=s"                          =>      "file that contain the true cis-elements",
	"similarity_score=s"                    =>      "score between the query and the cis-element",
	"match_length=s"                        =>      "lenght of the cis-element",
	"wildcard_penalty=s"                    =>      "are wildcard penaltiezed",
);
GetOptions(\%opt, "help!", keys %legend);

my($pattern_list)	= "";
my($place_file)		= "";
my($similarity_score)	= "";
my($match_length)	= "";
my($wildcard_penalty)	= "";

if (exists($opt{'pattern_list'})) { $pattern_list = $opt{'pattern_list'}; } else { die "Missing parameter: pattern_list"; }
if (exists($opt{'place_file'})) { $place_file = $opt{'place_file'}; }   else { die "Missing parameter: place_file"; }
if (exists($opt{'similarity_score'})) { $similarity_score = $opt{'similarity_score'}; } else { die "Missing parameter: similarity_score"; }
if (exists($opt{'match_length'})) { $match_length = $opt{'match_length'}; } else { die "Missing parameter: match_length"; }
if (exists($opt{'wildcard_penalty'})) { $wildcard_penalty = $opt{'wildcard_penalty'}; } else { die "Missing parameter: wildcard_penalty"; }

&main($pattern_list, $place_file, $similarity_score, $match_length, $wildcard_penalty);
exit;

##########################################################################################################
# main interface

sub main {
	my($pattern_list, $place_file, $similarity_score, $match_length, $wildcard_penalty) 	= @_;	
	my(%results)										= ();
	my(@screen_patterns)									= ();
	my(@data) 										= ();
	my(@print_columns)									= ();
	
	my($results_text)									= "";	
	my($query_pattern)									= "";
	my($query_len)										= "";
	my($len_error)										= "";
	my($wfont)										= "";
	
	my($print_header)									= 0;
	
	open(PATLIST, "$pattern_list") or die "$pattern_list file was not found";
	@screen_patterns = <PATLIST>;
	close PATLIST;
	
	open(DATA, "$place_file") or die "$place_file file was not found";
	@data = <DATA>;
	close DATA;
	
	$wfont = "<font color='#000000'>";
	
	for(my($g) = 0; $g < @screen_patterns; ++$g) {
		%results 	= ();
		$query_pattern 	= $screen_patterns[$g];
		$query_pattern 	=~ s/^\s+|\s+$//g;
		$query_len 	= length($query_pattern);
		$len_error	= $query_len*$match_length;
		$print_header 	= 1;

		if ($query_len > 0) {
			&PatternScoring($query_pattern, \@data, $similarity_score, $wildcard_penalty, $query_len, $len_error, \%results);
		
			print STDOUT "<table border='1' cellpadding='4' cellspacing='0'";
			print STDOUT "bordercolor='#00008B' align='center' bgcolor='#B0C4DE' width='95%'>\n";
			print STDOUT "<tr><td colspan='10' width='100%' align='center'>";
			print STDOUT "<font color = '#000000'><b>$query_pattern</b></font></td></tr>\n";
		
			foreach my $key (sort { $results{$b}{'score'} <=> $results{$a}{'score'}; } (keys(%results))) {
				if($print_header == 1) {
					print STDOUT "<tr>\n";
					print STDOUT "<td colspan='1' align='center' width='10%' >".$wfont."Ac</font></td>\n";
					print STDOUT "<td colspan='1' align='center' width='20%' >".$wfont."Id</font></td>\n";
					print STDOUT "<td colspan='1' align='center' width='15%' >".$wfont."Pattern (Subject)</font></td>\n";
					print STDOUT "<td colspan='1' align='center' width='10%' >".$wfont."Match (M)</font></td>\n";
					print STDOUT "<td colspan='1' align='center' width='10%' >".$wfont."Score (M/Q)</font></td>\n";
					print STDOUT "<td colspan='1' align='center' width='10%' >".$wfont."Score (M/S)</font></td>\n";
					print STDOUT "<td colspan='1' align='center' width='5%'  >".$wfont."Strand</font></td>\n";
					print STDOUT "<td colspan='1' align='center' width='5%'  >".$wfont."Pos</font></td>\n";
					print STDOUT "<td colspan='1' align='center' width='15%' >".$wfont."Match part</font></td>\n";
					print STDOUT "</tr>\n";
					$print_header = 0;
				}
				
				$results_text = $results{$key}{'text'};
				@print_columns = split(/\t/, $results_text);
				print STDOUT "<tr bgcolor = '#B0C4DE'>\n";
				print STDOUT "<td colspan='1' align='center' width='10%'>";
				
				if($print_columns[0] =~ m/^S/) {
					print STDOUT "<a href='http://www.dna.affrc.go.jp/PLACE/wais.html' target='_blank'>";
					print STDOUT "<font color='#00008B'>$print_columns[0]</font></a></td>\n";
				} elsif($print_columns[0] =~ m/^M0/) {
					print STDOUT "<a href='http://www.gene-regulation.com/cgi-bin/pub/databases/transfac/search.cgi' target='_blank'>";
					print STDOUT "<font color='#00008B'>$print_columns[0]</font></a></td>\n";
				} elsif($print_columns[0] =~ m/^MA/) {
					print STDOUT "<a href='http://mordor.cgb.ki.se/cgi-bin/jaspar2005/jaspar_db.pl' target='_blank'>";
					print STDOUT "<font color='#00008B'>$print_columns[0]</font></a></td>\n";
				}
				
				print STDOUT "<td colspan='1' align='center' width='20%'>$print_columns[1]</td>\n";
				print STDOUT "<td colspan='1' align='center' width='15%'>$print_columns[2]</td>\n";
				print STDOUT "<td colspan='1' align='center' width='10%'>$print_columns[3]</td>\n";
				print STDOUT "<td colspan='1' align='center' width='10%'>$print_columns[4]</td>\n";
				print STDOUT "<td colspan='1' align='center' width='10%'>$print_columns[5]</td>\n";
				print STDOUT "<td colspan='1' align='center' width='5%'>$print_columns[6]</td>\n";
				print STDOUT "<td colspan='1' align='center' width='5%'>$print_columns[7]</td>\n";
				print STDOUT "<td colspan='1' align='center' width='15%'>$print_columns[8]</td>\n";
				print STDOUT "</tr>\n";
			}

			if ($print_header == 1) {
				print STDOUT "<tr bgcolor = '#B0C4DE'>\n";
				print STDOUT "<td colspan='9' align='center' width='10%'>No matching patterns found ";
				print STDOUT "when using the given parameters</td>";
				print STDOUT "</tr>\n";
			}
			
			print STDOUT "</table><br>\n";
		}
	}
};
##########################################################################################################
# Patterns scoring
sub PatternScoring {
	my($query_pattern, $data, $similarity_score, $wildcard_penalty, $query_len, $len_error, $results) 	= @_;
	my(@columns)												= ();	

	my($line)												= "";	
	my($query_pattern_temp)											= "";
	my($header)												= "";
	my($match_pattern)											= "";
	my($match_temp1)											= "";
	my($match_temp2)											= "";
	my($char1)												= "";
	my($char2)												= "";
	my($show_patt)												= "";
	my($res_word) 												= "";
	my($query_strand)											= "";
	
	my($pq_score)												= 0;
	my($qp_score)												= 0;
	my($score)												= 0;
	my($match_len)												= 0;
	my($minimum_len)											= 0;
	
	for (my($i) = 0; $i < @$data; ++$i) {
		$line 		= @$data[$i];
		$line 		=~ s/^\s+|\s+$//g;
		@columns 	= split(/\s+/, $line);
	
		$header		= "$columns[4]\t$columns[1]\t";
		$match_pattern	= $columns[2];
		$match_pattern	=~ s/\s//g;
		$match_len	= length($match_pattern);
			
		if (($match_len > ($query_len - ($len_error))) && ($match_len < ($query_len + ($len_error)))) {
			$match_temp2 = "";
			for(my($j) = 0; $j < ($query_len-1); ++$j) {
				$match_temp2 .= "_";
			}
			$match_temp1 = $match_temp2.$match_pattern.$match_temp2;

			$query_pattern_temp = $query_pattern;
			for(my($k) = 0; $k < 2; ++$k) {
				$query_strand = "+";
				if($k == 1) {
					$query_pattern_temp = uc($query_pattern_temp);
					$query_pattern_temp = reverse($query_pattern_temp);
					$query_pattern_temp =~ tr/ACGTMRWSYKVHDB/TGCAKYWSRMBDHV/;
					$query_strand = "-";
				}
					
				for(my($j) = 0; $j < ($match_len + ($query_len-1)); ++$j) {
					$score = 0;
					for(my($k) = 0; $k < $query_len; ++$k) {
						$char1 = substr($query_pattern_temp, $k, 1);
						$char2 = substr($match_temp1, ($j + $k), 1);
						$score += &MatchCharacters($char1, $char2, $wildcard_penalty);
					}

					if ($query_len > $match_len) { 
						$minimum_len = $match_len;	
					} else {		
						$minimum_len = $query_len
					}

					if (($score / $minimum_len) >= $similarity_score) {
						$show_patt = substr($match_temp1, $j, $query_len);
						$pq_score = sprintf("%.2f", $score/$query_len);
						$qp_score = sprintf("%.2f", $score/$match_len);
						$score	  = sprintf("%.2f", $score);
						
						$res_word  = "$header";
						$res_word .= "$match_pattern\t$score\t$pq_score\t";
						$res_word .= "$qp_score\t$query_strand\t".($j-($query_len-1))."\t";
						$res_word .= "$show_patt";

						if (!(exists(${$results}{$header}{'score'}))) {
							${$results}{$header}{'score'} = $score / $minimum_len;
							${$results}{$header}{'text'}  = $res_word;
						} else {
							if (${$results}{$header}{'score'} < ($score / $minimum_len)) {
								${$results}{$header}{'score'} = $score / $minimum_len;
								${$results}{$header}{'text'}  = $res_word;
							}
						}
					}
				}
			}
		}
	}
};
##########################################################################################################
# Char scoring
sub MatchCharacters {
	my($char1, $char2, $wildcard_penalty) 	= @_;
        my($wildcard1)                          = 1;
	my($wildcard2)                          = 1;
	my($wildcard3)				= 1;
	my($wildcard4)				= 1;
	my($score)				= 0;
	
	if ($wildcard_penalty == 1) {
		$wildcard1      = (1/1);
		$wildcard2      = (1/2);
		$wildcard3      = (1/3);
		$wildcard4      = (1/4);
	}

	if ($char1 =~ m/$char2/i)                       		{ $score = $wildcard1; }
	elsif ($char1 =~ m/M/i && $char2 =~ m/[AC]/i )  		{ $score = $wildcard2; }
	elsif ($char1 =~ m/R/i && $char2 =~ m/[AG]/i )  		{ $score = $wildcard2; }
	elsif ($char1 =~ m/W/i && $char2 =~ m/[AT]/i )  		{ $score = $wildcard2; }
	elsif ($char1 =~ m/S/i && $char2 =~ m/[CG]/i )  		{ $score = $wildcard2; }
	elsif ($char1 =~ m/Y/i && $char2 =~ m/[CT]/i )  		{ $score = $wildcard2; }
	elsif ($char1 =~ m/K/i && $char2 =~ m/[GT]/i )  		{ $score = $wildcard2; }
	
	elsif ($char2 =~ m/M/i && $char1 =~ m/[AC]/i )  		{ $score = $wildcard2; }
	elsif ($char2 =~ m/R/i && $char1 =~ m/[AG]/i )  		{ $score = $wildcard2; }
	elsif ($char2 =~ m/W/i && $char1 =~ m/[AT]/i )  		{ $score = $wildcard2; }
	elsif ($char2 =~ m/S/i && $char1 =~ m/[CG]/i )  		{ $score = $wildcard2; }
	elsif ($char2 =~ m/Y/i && $char1 =~ m/[CT]/i )  		{ $score = $wildcard2; }
	elsif ($char2 =~ m/K/i && $char1 =~ m/[GT]/i )  		{ $score = $wildcard2; }
	
	elsif ($char1 =~ m/V/i && $char2 =~ m/[ACGMRS]/i ) 		{ $score = $wildcard3; }
	elsif ($char1 =~ m/H/i && $char2 =~ m/[ACTMWY]/i ) 		{ $score = $wildcard3; }
	elsif ($char1 =~ m/D/i && $char2 =~ m/[AGTRWK]/i ) 		{ $score = $wildcard3; }
	elsif ($char1 =~ m/B/i && $char2 =~ m/[CGTSYK]/i ) 		{ $score = $wildcard3; }
	
	elsif ($char2 =~ m/V/i && $char1 =~ m/[ACGMRS]/i ) 		{ $score = $wildcard3; }
	elsif ($char2 =~ m/H/i && $char1 =~ m/[ACTMWY]/i ) 		{ $score = $wildcard3; }
	elsif ($char2 =~ m/D/i && $char1 =~ m/[AGTRWK]/i ) 		{ $score = $wildcard3; }
	elsif ($char2 =~ m/B/i && $char1 =~ m/[CGTSYK]/i ) 		{ $score = $wildcard3; }
	
	elsif ($char1 =~ m/N/i && $char2 =~ m/[ACGTMRWSYKVHDBN]/i)     	{  $score = $wildcard4; }
	elsif ($char2 =~ m/N/i && $char1 =~ m/[ACGTMRWSYKVHDBN]/i)     	{  $score = $wildcard4; }

	return($score);	
};
##########################################################################################################

